from typing import Final

from wemake_python_styleguide.presets.topics import classes, complexity, naming
from wemake_python_styleguide.visitors.ast import (  # noqa: WPS235
    blocks,
    builtins,
    compares,
    conditions,
    decorators,
    exceptions,
    functions,
    imports,
    iterables,
    keywords,
    loops,
    modules,
    operators,
    pm,
    redundancy,
    statements,
    subscripts,
)
from wemake_python_styleguide.visitors.tokenize import (
    functions as tokenize_functions,
)

#: Used to store all general visitors to be later passed to checker:
PRESET: Final = (
    # General:
    statements.StatementsWithBodiesVisitor,
    statements.PointlessStarredVisitor,
    statements.WrongNamedKeywordVisitor,
    statements.AssignmentPatternsVisitor,
    statements.WrongMethodArgumentsVisitor,
    keywords.WrongRaiseVisitor,
    keywords.WrongKeywordVisitor,
    keywords.WrongContextManagerVisitor,
    keywords.ConsistentReturningVisitor,
    keywords.ConstantKeywordVisitor,
    keywords.GeneratorKeywordsVisitor,
    loops.WrongComprehensionVisitor,
    loops.WrongLoopVisitor,
    loops.WrongLoopDefinitionVisitor,
    loops.WrongStatementInLoopVisitor,
    functions.WrongFunctionCallVisitor,
    functions.FunctionDefinitionVisitor,
    functions.UselessLambdaDefinitionVisitor,
    functions.WrongFunctionCallContextVisitor,
    functions.FunctionSignatureVisitor,
    tokenize_functions.WrongEmptyLinesCountVisitor,
    exceptions.WrongTryExceptVisitor,
    exceptions.NestedTryBlocksVisitor,
    exceptions.WrongExceptHandlerVisitor,
    imports.WrongImportVisitor,
    builtins.WrongNumberVisitor,
    builtins.WrongStringVisitor,
    builtins.WrongFormatStringVisitor,
    builtins.WrongAssignmentVisitor,
    builtins.WrongCollectionVisitor,
    operators.UselessOperatorsVisitor,
    operators.WrongMathOperatorVisitor,
    operators.WalrusVisitor,
    compares.WrongConditionalVisitor,
    compares.CompareSanityVisitor,
    compares.WrongConstantCompareVisitor,
    compares.InCompareSanityVisitor,
    compares.WrongFloatComplexCompareVisitor,
    conditions.IfStatementVisitor,
    conditions.BooleanConditionVisitor,
    conditions.MatchVisitor,
    conditions.ChainedIsVisitor,
    iterables.IterableUnpackingVisitor,
    blocks.AfterBlockVariablesVisitor,
    subscripts.SubscriptVisitor,
    subscripts.ImplicitDictGetVisitor,
    subscripts.CorrectKeyVisitor,
    subscripts.StrictSliceOperations,
    decorators.WrongDecoratorVisitor,
    redundancy.RedundantEnumerateVisitor,
    pm.MatchSubjectVisitor,
    # Modules:
    modules.EmptyModuleContentsVisitor,
    modules.MagicModuleFunctionsVisitor,
    modules.ModuleConstantsVisitor,
    # Topics:
    *complexity.PRESET,
    *naming.PRESET,
    *classes.PRESET,
)
